require 'freshdesk/api/v2/util/client'
require 'httparty'

module Freshdesk
  class Base
    attr_accessor :domain, :apikey

    def initialize(attrs = {})
      @attrs = attrs.dup
      @attrs.each do |key,value|
        self.instance_variable_set("@#{key}".to_sym, value)
      end
    end

    def connected?
      response = client.get("tickets")
      response.success?
    end

    def all(options = {})
      items = []
      link = nil
      page = options[:page] || 1
      per_page = options[:per_page] || 100
      begin
        response = client.get("/#{resource_name}",
          options.merge(:page => page, :per_page => per_page))
        link = response.headers['link']
        response.parsed_response.map { |source| items.push(self.class.new(source)) } if response.success?
        page += 1
      end until link.nil?
      items
    end

    def create(options = {},body = {})
      response = client.post("/#{resource_name}", options, body)
      self.class.new(response) if response.success?
    end

    def find(id, options = {})
      response = client.get("/#{resource_name}/#{id}", options)
      self.class.new(response) if response.success?
    end

    def update(id, options = {})
      client.class.put("/#{resource_name}/#{id}", {}, options)
    end

    def delete(id)
      response = client.delete("/#{resource_name}/#{id}")
      response.success?
    end

    def delete_all
      all.each {|s| delete(s.id) }
    end


    private
    def resource_name
      class_name = self.class.name.split("::")
      class_name.last(class_name.size-1).join('/').downcase
    end

    def client
      @client ||= Util::Client.new(@attrs)
    end
  end
end
