module Freshdesk
  module Request
    def get(path, options = {}, body = {})
      self.class.get(prepare_path(path), prepare_options(options, body))
    end

    def put(path, options = {}, body = {})
      self.class.put(prepare_path(path), prepare_options(options, body))
    end

    def post(path, options = {}, body = {})
      handle_response self.class.post(prepare_path(path), prepare_options(options, body))
    end

    def delete(path, options = {}, body = {})
      self.class.delete(prepare_path(path), prepare_options(options, body))
    end

    private
    def prepare_options(options, body)
      if body.empty?
        { :query => options, :basic_auth => credentials }
      else
        {
          :headers => {'content-type' => 'application/json'},
          :query => options,
          :body => body.to_json,
          :basic_auth => credentials
        }
      end
    end

    def prepare_path(path)
      "https://#{@domain}.freshdesk.com/api/v2/#{path}"
    end

    def handle_response(response)
      !response.success? and raise UnsuccessfulRequestError, response.inspect
      response
    end
  end
end
